//
//  AppDelegate.m
//  Appear2mee
//
//  Created on 31/01/19.
//  Copyright © 2019 2mee Ltd. All rights reserved.
//

#import <AVFoundation/AVFoundation.h>
#import "AppDelegate.h"
#import <Appear2mee/Appear2mee.h>

#import "Appear2meeConstants.h"
@import UserNotifications;

@interface AppDelegate ()<UNUserNotificationCenterDelegate>

@end

@implementation AppDelegate

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions {
    
    [Appear2mee initWithID:TWOMEE_CLIENT_ID clientKey:TWOMEE_SECRET_KEY usingAppGroup:APPGROUP];

    //These are the default values. If you need to change something uncomment
    NSDictionary *appear2meeDefaultSettings =
    @{
      kAppear2meeNotificationDismiss:@"Dismiss",
      // ios 12 will use custom Action.
      kAppear2meeNotificationAccept:@"OK",
      // ios 12 will use touch image/video to play.
      kAppear2meeNotificationPlay:@"Play",
     
      
      kAppear2meeMessageExpired:@"Sorry, opportunity missed!",
      
      // Position of face in app.
      //kAppear2meeFaceHeadPosition:kAppear2meeFaceHeadTopLeft,
      //kAppear2meeFaceShouldersPosition:kAppear2meeFaceShouldersRight,
     
      // Color of app display background when showing an in app notification.
      kAppear2meeDisplayBackgroundColor:[Appear2mee rgba:[
                                                          UIColor colorWithRed:0.0 green:0.0 blue:0.0 alpha:0.6]],
      kAppear2meeFaceDisplayBackgroundColor:[Appear2mee rgba:[
                                                               UIColor colorWithRed:0.0 green:0.0 blue:0.0 alpha:0.2]],
     
     // Color of face background, only used with kAppear2meeFaceHeadCenter setting for kAppear2meeFaceHeadPosition:
     kAppear2meeFaceCenterBackgroundColor:[Appear2mee rgba:[
     UIColor colorWithRed:1.0 green:1.0 blue:1.0 alpha:1.0]],
      
      kAppear2meeFaceAlertDismissTime:@5.0,
     
     // Notification sounds. Defaults are built into the SDK.
     // If you need to change uncomment.
     // kAppear2meeFaceNotificationSound:@"tothepoint.aiff",
     // kAppear2meeNotificationSound:@"slowspringboard.aiff",
      
      // Content Extension background images. Defaults are built into the SDK.
      // These need to be stored in the Content Extension bundle.
      // If you need to change uncomment.
      //kAppear2meeFaceContentExtensionBackgroundImage:@"face.png",
      //kAppear2meeContentExtensionBackgroundImage:@"media.jpg",
     
      };
    [Appear2mee defaultSettings:appear2meeDefaultSettings]; 
     
    
    
    // If you want just to display notifications in the notification center, even when app is active,
    // use DISPLAY_NOTIFICATION_WHEN_ACTIVE = true
    // This is the recommended setting
    self.USE_NOTIFICATION_CENTER_WHEN_ACTIVE=true;
    
    [self requestPermissionForNotifications];
   
    return YES;
}


- (void)applicationWillResignActive:(UIApplication *)application {
    // Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
    // Use this method to pause ongoing tasks, disable timers, and invalidate graphics rendering callbacks. Games should use this method to pause the game.
  
}


- (void)applicationDidEnterBackground:(UIApplication *)application {
    // Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later.
    // If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
}


- (void)applicationWillEnterForeground:(UIApplication *)application {
    // Called as part of the transition from the background to the active state; here you can undo many of the changes made on entering the background.
}


- (void)applicationDidBecomeActive:(UIApplication *)application {
    // Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
}


- (void)applicationWillTerminate:(UIApplication *)application {
    // Called when the application is about to terminate. Save data if appropriate. See also applicationDidEnterBackground:.
}

-(void) requestPermissionForNotifications {
    UNUserNotificationCenter *center = [UNUserNotificationCenter currentNotificationCenter];
    center.delegate = self;
    
    UNAuthorizationOptions options;
    if (@available(iOS 12.0, *)) {
        options = (UNAuthorizationOptionSound | UNAuthorizationOptionAlert| UNAuthorizationOptionBadge
                    | UNAuthorizationOptionProvisional  // include for no permission dialog
                   );
    } else {
        options = (UNAuthorizationOptionSound | UNAuthorizationOptionAlert| UNAuthorizationOptionBadge);
    }
    [center requestAuthorizationWithOptions:options
                          completionHandler:^(BOOL granted, NSError * _Nullable error)
     {
         if( !error ) {
             if(granted) {
                 // Setup Appear2mee notification actions.
                 // Add any other independent UNNotificationCategory before
                 // calling this method
                 [Appear2mee addAppear2meeNotificationCategory];
                 dispatch_async(dispatch_get_main_queue(), ^{
                     [[UIApplication sharedApplication] registerForRemoteNotifications];
                 });
             }
         }
     }];
}

- (NSString *)stringWithDeviceToken:(NSData *)deviceToken {
    const char *data = [deviceToken bytes];
    NSMutableString *token = [NSMutableString string];

    for (NSUInteger i = 0; i < [deviceToken length]; i++) {
        [token appendFormat:@"%02.2hhx", data[i]];
    }
    return [token copy];
}

- (void)application:(UIApplication *)application didRegisterForRemoteNotificationsWithDeviceToken:(NSData *)deviceToken
{
    NSString *token = [self stringWithDeviceToken:deviceToken];
    NSLog(@"deviceToken %@", token);
    [Appear2mee registerDeviceToken:deviceToken];
}

- (void)application:(UIApplication *)application didReceiveRemoteNotification:(NSDictionary *)userInfo fetchCompletionHandler:(void (^)(UIBackgroundFetchResult result))completionHandler {
    //get custom data from push notification
    if([Appear2mee didReceiveRemoteNotification:userInfo fetchCompletionHandler:completionHandler]) {
        // Appear2meeee will handle this notification
        
    } else {
        // is some other type of notification
        completionHandler(UIBackgroundFetchResultNoData);
    }
}


#pragma mark - UNUserNotificationCenter Delegate // >= iOS 10

- (void)userNotificationCenter:(UNUserNotificationCenter *)center willPresentNotification:(UNNotification *)notification withCompletionHandler:(void (^)(UNNotificationPresentationOptions options))completionHandler{
    if([Appear2mee isAppear2meeNotification:notification])  {
        NSDictionary *keyVals = [Appear2mee notificationKeyValues:notification];
        NSLog(@"keyValues %@",keyVals);
    }
    
    if([Appear2mee isAppear2meeNotification:notification]) { // notification from 2mee Exchange
        if(self.USE_NOTIFICATION_CENTER_WHEN_ACTIVE) {
            completionHandler(UNNotificationPresentationOptionAlert | UNNotificationPresentationOptionBadge | UNNotificationPresentationOptionSound);
        } else {
            [Appear2mee displayNotification:notification withAcceptAction:^(NSDictionary *exchangeKeyValues)
            {
                NSLog(@"accepted");
            } withDismissAction:^(NSDictionary *exchangeKeyValues)
            {
                 NSLog(@"dismissed");
            }];
            completionHandler(UNNotificationPresentationOptionNone);
        }
    } else {
        // handle other notifications
        completionHandler(UNNotificationPresentationOptionAlert | UNNotificationPresentationOptionBadge | UNNotificationPresentationOptionSound);
    }
}

- (void)userNotificationCenter:(UNUserNotificationCenter *)center didReceiveNotificationResponse:(UNNotificationResponse *)response withCompletionHandler:(void(^)(void))completionHandler __IOS_AVAILABLE(10.0) __WATCHOS_AVAILABLE(3.0) __TVOS_PROHIBITED
{
    if([Appear2mee isAppear2meeNotificationResponse:response])  {
        NSDictionary *keyVals = [Appear2mee notificationResponseKeyValues:response];
        NSLog(@"keyValues %@",keyVals);

        NSString *presentation = keyVals[@"presentation"];
        NSDictionary *appear2meeDefaultSettings =
        @{kAppear2meeFaceDisplayBackgroundColor:[Appear2mee rgba:[
                                                                   UIColor colorWithRed:0.0 green:0.0 blue:0.0 alpha:0.2]],
        };
        if(presentation != nil && [presentation isEqualToString:@"dark"]) {
            appear2meeDefaultSettings =
            @{kAppear2meeFaceDisplayBackgroundColor:[Appear2mee rgba:[
                                                                       UIColor colorWithRed:0.0 green:0.0 blue:0.0 alpha:0.8]],
            };
        }
        [Appear2mee defaultSettings:appear2meeDefaultSettings];
    }
    if([Appear2mee didReceiveNotificationResponse:response withAcceptAction:^(NSDictionary *exchangeKeyValues) {
        NSLog(@"Accept action");
        if(exchangeKeyValues[@"link"]) {
            NSLog(@"value %@",exchangeKeyValues[@"link"]);
        }
    } withDismissAction:^(NSDictionary *exchangeKeyValues) {
        NSLog(@"Dismiss action");
    }])
    {
       // it was a 2mee Exchange notification
    } else {
        // some other notification
    }
    
    completionHandler();
}

@end
