//
//  CarListViewController.m
//  AddItemTableViewDemo
//
//  Created by Arthur Knopper on 15-06-13.
//  Copyright (c) 2013 Arthur Knopper. All rights reserved.
//  My code does not apply any license ...you can use it anyway .
//  MIT
//  Much modified Copyright (c) 2018 2Mee Ltd

#import "TagListViewController.h"

@interface TagListViewController ()<UIAdaptivePresentationControllerDelegate>{
    NSString *editableKey;
}

@property (nonatomic, strong) NSMutableDictionary *tagsDict;
@property (nonatomic, strong) NSMutableArray *tags;

@end

@implementation TagListViewController

- (id)initWithStyle:(UITableViewStyle)style
{
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
    }
    return self;
}


- (void)viewDidLoad
{
    [super viewDidLoad];
   
    self.tagsDict = [[Appear2mee tags] mutableCopy];
    
    self.tags = [NSMutableArray new];
    for(NSString *key in self.tagsDict) {
        [self.tags addObject:key];
    }
    NSOrderedSet *orderedSet = [NSOrderedSet orderedSetWithArray:self.tags];
    self.tags = [[orderedSet array] mutableCopy];

    [self.tableView reloadData];
    
    if(self.navigationController) {
        self.navigationController.presentationController.delegate = self;
    } else {
        self.presentationController.delegate = self;
    }

}

-(void) saveChanges {
   [Appear2mee registerTags:self.tagsDict];
}

// Catch user dismisal of tags
- (void)presentationControllerWillDismiss:(UIPresentationController *)presentationController {
    UIViewController *tagViewController;
    
    id navortag = presentationController.presentedViewController;
    if( [navortag isKindOfClass:[UINavigationController class] ]) {
        UINavigationController *navController = navortag;
        tagViewController = [navController.viewControllers firstObject];
    } else {
        tagViewController = navortag;
    }
    if( [tagViewController isKindOfClass:[TagListViewController class] ]) {
        [(TagListViewController *)tagViewController saveChanges];
    }
    
}

- (IBAction)doneAction:(id)sender {
    [Appear2mee registerTags:self.tagsDict];
    [self dismissViewControllerAnimated:YES completion:nil];
}


- (IBAction)cancelTag:(UIStoryboardSegue *)segue
{
}

- (IBAction) saveTag:(UIStoryboardSegue *)segue
{
    TagDetailViewController *tagDetailVC = segue.sourceViewController;
    
    if([tagDetailVC.tagKey length] == 0 || [tagDetailVC.tagValue length] == 0) {
        return;
    } else {
        BOOL valid;
        id value;
        NSCharacterSet *alphaNums = [NSCharacterSet decimalDigitCharacterSet];
        NSCharacterSet *inStringSet = [NSCharacterSet characterSetWithCharactersInString:tagDetailVC.tagValue];
        valid = [alphaNums isSupersetOfSet:inStringSet];

        if (!valid) {
            value = tagDetailVC.tagValue;
        } else {
            NSNumberFormatter *f = [[NSNumberFormatter alloc] init];
            f.numberStyle = kCFNumberFormatterNoStyle;
            NSNumber *number = [f numberFromString:tagDetailVC.tagValue];
            value = number;
        }
        //editing
        if (tagDetailVC.isEditingMode) {
            if (editableKey) {
                [self.tagsDict removeObjectForKey:editableKey];
                [self.tags removeObject:editableKey];
                editableKey = nil;
            }
        }
        
        [self.tagsDict setObject:value forKey:tagDetailVC.tagKey];
        [self.tags addObject:tagDetailVC.tagKey];
        NSOrderedSet *orderedSet = [NSOrderedSet orderedSetWithArray:self.tags];
        self.tags = [[orderedSet array] mutableCopy];
        
        [self.tableView reloadData];
    }
}


- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    // Return the number of sections.
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    // Return the number of rows in the section.
    return [self.tags count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *CellIdentifier = @"TagCell";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier forIndexPath:indexPath];
    
    NSString *key = [self.tags objectAtIndex:indexPath.row];
    id value = [self.tagsDict objectForKey:key];
    cell.textLabel.text = [NSString stringWithFormat:@"%@:%@", key, value];
        
    return cell;
}


// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the specified item to be editable.
    return YES;
}

// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        NSString *key =self.tags[indexPath.row];
        self.tagsDict[key] = nil;
        [self.tags removeObjectAtIndex:indexPath.row];
        
        // Delete the row from the data source
        [tableView deleteRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationFade];
    }   
    else if (editingStyle == UITableViewCellEditingStyleInsert) {
        // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view
    }   
}


/*
// Override to support rearranging the table view.
- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath
{
}
*/

/*
// Override to support conditional rearranging of the table view.
- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the item to be re-orderable.
    return YES;
}
*/

#pragma mark - Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
   
}
#pragma mark - segue
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    // Make sure your segue name in storyboard is the same as this line
    if ([[segue identifier] isEqualToString:@"editTagInfo"])
    {
        //if you need to pass data to the next controller do it here
        TagDetailViewController *tagDetailVC = (TagDetailViewController *)segue.destinationViewController;
        NSIndexPath *indexPath = [self.tableView indexPathForCell:sender];
        
        NSString *key = [self.tags objectAtIndex:indexPath.row];
        id value = [self.tagsDict objectForKey:key];
        
        tagDetailVC.tagKey = key;
        tagDetailVC.tagValue = value;
        tagDetailVC.isEditingMode = YES;
        
        editableKey = key;
    }
}


@end
