//
//  ViewController.m
//  Appear2meeSimple
//
//  Created on 31/01/19.
//  Copyright © 2019 2mee Ltd. All rights reserved.
//

#import "ViewController.h"
#import "TagListViewController.h"
#import "AppDelegate.h"
#import <Appear2mee/Appear2mee.h>
#import <UserNotifications/UserNotifications.h>

@interface ViewController ()
@property (weak, nonatomic) IBOutlet UISwitch *onlyWifiPermissionSwitch;
@property (weak, nonatomic) IBOutlet UISwitch *downloadPermissionSwitch;
@property (weak, nonatomic) IBOutlet UISwitch *servicePermissionSwitch;
@property (weak, nonatomic) IBOutlet UISwitch *soundOnMutePermissionSwitch;
//@property (weak, nonatomic) IBOutlet MIBadgeButton *inBoxBadgeButton;

@property (weak, nonatomic) IBOutlet UISwitch *useNotificationCenterSwitch;
@property (strong, nonatomic) IBOutlet UITapGestureRecognizer *singleTap;
@property (strong, nonatomic) IBOutlet UITapGestureRecognizer *doubleTap;

@end

static int msgCount;// used to inc message title

@implementation ViewController

- (void)viewDidLoad {
    [super viewDidLoad];
}

-(void) dealloc {
     [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

-(void) viewWillAppear: (BOOL) animated {
    [super viewWillAppear:animated];
    [self.navigationController setNavigationBarHidden:YES animated:animated];
    AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
    self.useNotificationCenterSwitch.on = appDelegate.USE_NOTIFICATION_CENTER_WHEN_ACTIVE;
    self.onlyWifiPermissionSwitch.on = [Appear2mee hasWifiOnlyPermission];
    self.downloadPermissionSwitch.on = [Appear2mee hasDownloadPermission];
    self.servicePermissionSwitch.on = [Appear2mee hasServicePermission];
    self.soundOnMutePermissionSwitch.on = [Appear2mee hasSoundOnMutePermission];
}

#pragma mark Buttons/Switches

- (IBAction)allowDownloadAction:(UISwitch *)sender {
    [Appear2mee downloadPermission:sender.on];
}

- (IBAction)allowSoundOnMuteAction:(UISwitch *)sender {
    [Appear2mee soundOnMutePermission:sender.on];
}

- (IBAction)allowServiceAction:(UISwitch *)sender {
    [Appear2mee servicePermission:sender.on];
}

- (IBAction)onlyWifiDownload:(UISwitch *)sender {
    [Appear2mee wifiOnlyPermission:sender.on];
}

- (IBAction) useNotificationCenter:(UISwitch *)aswitch {
    AppDelegate *appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
    appDelegate.USE_NOTIFICATION_CENTER_WHEN_ACTIVE = aswitch.on;
}


- (IBAction)tappedOnButton:(id)sender {
    UIStoryboard *storyboard = [UIStoryboard storyboardWithName:@"Tags" bundle:nil];
    UIViewController *myVC = (UIViewController *)[storyboard instantiateInitialViewController];
    [self presentViewController:myVC animated:YES completion:nil];
}


- (IBAction)tappedOnUserIDButton:(id)sender {
    
    UIAlertController *alert = [UIAlertController alertControllerWithTitle:@"Link UserID"
                                                                   message:nil
                                                            preferredStyle:UIAlertControllerStyleAlert];
    [alert addTextFieldWithConfigurationHandler:^(UITextField *textField) {
        textField.keyboardType = UIKeyboardTypeAlphabet;
    }];
    
    UIAlertAction *okAction = [UIAlertAction actionWithTitle:@"OK"
                                                       style:UIAlertActionStyleDefault
                                                     handler:^(UIAlertAction *action) {
                                                         UITextField *textField = [alert.textFields firstObject];
                                                         [Appear2mee registerUserID:textField.text];
                                                     }];
    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:@"Cancel"
                                                           style:UIAlertActionStyleDefault
                                                         handler:^(UIAlertAction *action) {
                                                             [alert dismissViewControllerAnimated:YES completion:nil];
                                                         }];
    [alert addAction:okAction];
    [alert addAction:cancelAction];
    
    [self presentViewController:alert animated:YES completion:nil];
    
}

#pragma mark InApp message

- (IBAction)inAppMsg:(id)sender {
    [Appear2mee getMessage:@"Welcome Message" callback:^(NSDictionary * _Nullable item) {
        if(!item) return;
        
        [Appear2mee displayMessage:item withAcceptAction:^(NSDictionary * _Nonnull dict) {
            
        } withDismissAction:^(NSDictionary * _Nonnull dict) {
            
        }];
    }];
}


#pragma mark Test messages

-(NSString *) nowTimePlus:(double) secs {
    NSDate *now = [NSDate date];
    NSDate* newDate = [now dateByAddingTimeInterval:secs];
    NSDateFormatter *dateFormat = [[NSDateFormatter alloc] init];
    [dateFormat setTimeZone:[NSTimeZone timeZoneWithAbbreviation:@"UTC"]];
    [dateFormat setDateFormat:@"yyyy-MM-dd'T'HH:mm:ss.SSS'Z'"];
    NSString *dateString = [dateFormat stringFromDate:newDate];
    return dateString;
}

- (IBAction) testFacee:(id)sender {
    
    NSString *playDate = [self nowTimePlus:10.0]; // one min ago.
    static BOOL toggle=false;
    toggle= toggle?false:true;
    
    NSString *jobid;
    
    NSString *url,*about,*action;
    if(toggle) {
        url =@"https://faceestock.s3.eu-geo.objectstorage.softlayer.net/05fe4979-d80a-41ff-bcb1-297a7dc2704c.mp4";
        about=@"[{\"subject\":\"Not Funny\"},{\"page\":\"Not Funny\"},{\"link\":\"Not Funny\"}]";
        jobid = @"a60730a9555731f257f3b00f09890485_2ee";
        action = @"Hear a Joke";
    } else {
        url=@"https://exchange.cdnedge.bluemix.net/8540927e-8215-4126-af31-cb2ad9a76374-f959ffff-e026-4b05-ba07-8284be57e2bf-take-1.mp4";
        about=@"[{\"subject\":\"Funny\"},{\"page\":\"Not Funny\"},{\"link\":\"Funny\"},{\"presentation\":\"dark\"}]";
        jobid = @"1e19a1f0c2105be90d4fc75b7303ade3_cro";
        action = @"Have a bet";
    }
    
    msgCount++;
    
    NSDictionary *payload =
    @{@"payload":@{@"fm":@{@"action":action,
                           @"about":about,
                           @"title": [NSString stringWithFormat:@"A Title %d", msgCount],
                           @"subtitle":@"More Title Information",
                           @"url":url,
                           @"jobId":jobid,
                           @"message":@"One more test message",@"playTime":playDate,@"expiryTime":@120}
                   }
      };
    
    [Appear2mee testNotification:payload];
}

- (IBAction) testRichText:(id)sender {
    
    NSString *playDate = [self nowTimePlus:-60.0]; // one min ago.
    static BOOL toggle=false;
    toggle= !toggle;
    static BOOL toggle2=false;
    
    NSString *jobid;
    
    NSString *path,*key,*about,*action;
    if(toggle) {
        toggle2= !toggle2;
        if(toggle2) {
            key = @"img";
            path = @"https://media.wired.com/photos/5b45021f3808c83da3503cc7/master/w_600,c_limit/tumblr_inline_mjx5ioXh8l1qz4rgp.gif";
            about=@"[{\"subject\":\"Not Funny\"},{\"page\":\"Not Funny\"},{\"link\":\"Not Funny\"}]";
            jobid = @"richa60730a9555731f257f3b00f09890485_2ee";
            action = @"Hear a Joke";
        } else {
            key = @"img";
            path=@"https://www.freeiconspng.com/uploads/png-file-type-icon-0.png";
            about=@"[{\"subject\":\"Funny\"},{\"page\":\"Not Funny\"},{\"link\":\"Funny\"}]";
            jobid = @"rich1e19a1f0c2105be90d4fc75b7303ade3_cro";
            action = @"Have a bet";
        }
    } else {
        if(toggle2) {
            key = @"vid";
            path = @"https://sample-videos.com/video123/mp4/720/big_buck_bunny_720p_1mb.mp4";
            about=@"[{\"subject\":\"Not Funny\"},{\"page\":\"Not Funny\"},{\"link\":\"Not Funny\"}]";
            jobid = @"richa60730a9555731f257f3b00f09890485_2ee";
            action = @"Hear a Joke";
        } else {
            key = @"aud";
            path=@"https://sample-videos.com/audio/mp3/crowd-cheering.mp3";
            about=@"[{\"subject\":\"Funny\"},{\"page\":\"Not Funny\"},{\"link\":\"Funny\"}]";
            jobid = @"rich1e19a1f0c2105be90d4fc75b7303ade3_cro";
            action = @"Have a bet";
        }
    }
    
    msgCount++;
    NSDictionary *payload =
    @{@"payload":@{@"fm":@{@"action":action,
                           @"about":about,
                           @"title": [NSString stringWithFormat:@"A Title %d", msgCount],
                           @"subtitle":@"More Title Information",
                           key:path,
                           @"jobId":jobid,
                           @"message":@"One more test message",@"playTime":playDate,@"expiryTime":@120}
                   }
      };
    
    [Appear2mee testNotification:payload];
}


-(void)normalPushNotificationWithTitle:(NSString *)title subTitle:(NSString *)subTitle body:(NSString *)body identifier:(NSString *)identifier soundName:(NSString *)soundName timeInterval:(NSInteger)timeInterval repeat:(BOOL)repeat
{
    UNMutableNotificationContent *content = [[UNMutableNotificationContent alloc] init];
    content.title = title;
    content.subtitle = subTitle;
    content.body = body;
    content.badge = @1;
    
    if (soundName) {
        content.sound = [UNNotificationSound soundNamed:soundName];
    }else{
        content.sound = [UNNotificationSound defaultSound];
    }
    
    UNTimeIntervalNotificationTrigger *trigger = [UNTimeIntervalNotificationTrigger triggerWithTimeInterval:timeInterval repeats:repeat];
    
    [self registerPushNotificationWithIdentifier:identifier content:content trigger:trigger];
}

-(void)registerPushNotificationWithIdentifier:(NSString *)identifier content:(UNMutableNotificationContent *)content trigger:(UNNotificationTrigger *)trigger
{
    UNNotificationRequest *request;
    if ([trigger isKindOfClass:[UNCalendarNotificationTrigger class]]) {
        request = [UNNotificationRequest requestWithIdentifier:identifier content:content trigger:trigger];
    }else if ([trigger isKindOfClass:[UNTimeIntervalNotificationTrigger class]]){
        request = [UNNotificationRequest requestWithIdentifier:identifier content:content trigger:trigger];
    }else if ([trigger isKindOfClass:[UNLocationNotificationTrigger class]]){
        request = [UNNotificationRequest requestWithIdentifier:identifier content:content trigger:trigger];
    }
    
    // `push notification` ->`发送推送`
    UNUserNotificationCenter *center = [UNUserNotificationCenter currentNotificationCenter];
    [center addNotificationRequest:request withCompletionHandler:^(NSError * _Nullable error) {
        if (error == nil) {
            NSLog(@"the push notification identifier is: %@ \n and content is:  %@ \n with trigger: %@",request.identifier,content,trigger);
        }
    }];
}
@end
